@@ -94,6 +94,12 @@ static const enumdef orient_defs[] = {
     { NULL, }
 };
 
+
+#if PY_MAJOR_VERSION < 3
+struct module_state zbar_state;
+#endif
+
+
 int
 object_to_bool (PyObject *obj,
                 int *val)
@@ -119,7 +125,11 @@ parse_dimensions (PyObject *seq,
         PyObject *dim = PySequence_GetItem(seq, i);
         if(!dim)
             return(-1);
+#if PY_MAJOR_VERSION >= 3
+        *dims = PyLong_AsSsize_t(dim);
+#else
         *dims = PyInt_AsSsize_t(dim);
+#endif
         Py_DECREF(dim);
         if(*dims == -1 && PyErr_Occurred())
             return(-1);
@@ -127,13 +137,6 @@ parse_dimensions (PyObject *seq,
     return(0);
 }
 
-PyObject *zbar_exc[ZBAR_ERR_NUM];
-zbarEnumItem *color_enum[2];
-zbarEnum *config_enum;
-zbarEnum *modifier_enum;
-PyObject *symbol_enum;
-zbarEnumItem *symbol_NONE;
-zbarEnum *orient_enum;
 
 static PyObject*
 version (PyObject *self,
@@ -182,15 +185,56 @@ static PyMethodDef zbar_functions[] = {
     { NULL, },
 };
 
+
+#if PY_MAJOR_VERSION >= 3
+
+static int zbar_traverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(GETSTATE(m)->zbar_exc[0]);
+    return 0;
+}
+
+static int zbar_clear(PyObject *m) {
+    Py_CLEAR(GETSTATE(m)->zbar_exc[0]);
+    return 0;
+}
+
+
+struct PyModuleDef zbar_moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "zbar",
+    NULL,
+    sizeof(struct module_state),
+    zbar_functions,
+    NULL,
+    zbar_traverse,
+    zbar_clear,
+    NULL
+};
+
+#define INITERROR return NULL
+
 PyMODINIT_FUNC
-initzbar (void)
+PyInit_zbar(void)
+
+#else
+#define INITERROR return
+
+PyMODINIT_FUNC
+initzbar(void)
+#endif
 {
+#if PY_MAJOR_VERSION >= 3
     /* initialize types */
+    zbarEnumItem_Type.tp_base = &PyLong_Type;
+#else
     zbarEnumItem_Type.tp_base = &PyInt_Type;
     zbarException_Type.tp_base = (PyTypeObject*)PyExc_Exception;
 
-    if(PyType_Ready(&zbarException_Type) < 0 ||
-       PyType_Ready(&zbarEnumItem_Type) < 0 ||
+    if(PyType_Ready(&zbarException_Type) < 0)
+        INITERROR;
+#endif
+
+    if(PyType_Ready(&zbarEnumItem_Type) < 0 ||
        PyType_Ready(&zbarEnum_Type) < 0 ||
        PyType_Ready(&zbarImage_Type) < 0 ||
        PyType_Ready(&zbarSymbol_Type) < 0 ||
@@ -200,23 +244,34 @@ initzbar (void)
        PyType_Ready(&zbarImageScanner_Type) < 0 ||
        PyType_Ready(&zbarDecoder_Type) < 0 ||
        PyType_Ready(&zbarScanner_Type) < 0)
-        return;
+        INITERROR;
+
+    /* initialize module */
+#if PY_MAJOR_VERSION >= 3
+    PyObject *mod = PyModule_Create(&zbar_moduledef);
+#else
+    PyObject *mod = Py_InitModule("zbar", zbar_functions);
+#endif
+    if(!mod)
+        INITERROR;
+
+#if PY_MAJOR_VERSION >= 3
+    if(PyState_AddModule(mod, &zbar_moduledef)) {
+        Py_DECREF(mod);
+        INITERROR;
+    }
+#endif
+
+    struct module_state *st = GETSTATE(mod);
 
     /* initialize constant containers */
-    config_enum = zbarEnum_New();
-    modifier_enum = zbarEnum_New();
-    symbol_enum = PyDict_New();
-    orient_enum = zbarEnum_New();
-    if(!config_enum || !modifier_enum || !symbol_enum || !orient_enum)
-        return;
-
-    zbar_exc[0] = (PyObject*)&zbarException_Type;
-    zbar_exc[ZBAR_ERR_NOMEM] = NULL;
-    zbar_error_t ei;
-    for(ei = ZBAR_ERR_INTERNAL; ei < ZBAR_ERR_NUM; ei++) {
-        zbar_exc[ei] = PyErr_NewException(exc_names[ei], zbar_exc[0], NULL);
-        if(!zbar_exc[ei])
-            return;
+    st->config_enum = zbarEnum_New();
+    st->modifier_enum = zbarEnum_New();
+    st->symbol_enum = PyDict_New();
+    st->orient_enum = zbarEnum_New();
+    if(!st->config_enum || !st->modifier_enum || !st->symbol_enum || !st->orient_enum) {
+        Py_DECREF(mod);
+        INITERROR;
     }
 
     /* internally created/read-only type overrides */
@@ -224,17 +279,33 @@ initzbar (void)
     zbarEnum_Type.tp_setattr = NULL;
     zbarEnum_Type.tp_setattro = NULL;
 
-    /* initialize module */
-    PyObject *mod = Py_InitModule("zbar", zbar_functions);
-    if(!mod)
-        return;
+    /* zbar internal exception objects */
+#if PY_MAJOR_VERSION >= 3
+    st->zbar_exc[0] = PyErr_NewException("zbar.Exception", NULL, NULL);
+#else
+    st->zbar_exc[0] = (PyObject*)&zbarException_Type;
+#endif
+    if (st->zbar_exc[0] == NULL) {
+        Py_DECREF(mod);
+        INITERROR;
+    }
+
+    st->zbar_exc[ZBAR_ERR_NOMEM] = NULL;
+    zbar_error_t ei;
+    for(ei = ZBAR_ERR_INTERNAL; ei < ZBAR_ERR_NUM; ei++) {
+        st->zbar_exc[ei] = PyErr_NewException(exc_names[ei], st->zbar_exc[0], NULL);
+        if (!st->zbar_exc[ei]) {
+            Py_DECREF(mod);
+            INITERROR;
+        }
+    }
 
     /* add types to module */
     PyModule_AddObject(mod, "EnumItem", (PyObject*)&zbarEnumItem_Type);
     PyModule_AddObject(mod, "Image", (PyObject*)&zbarImage_Type);
-    PyModule_AddObject(mod, "Config", (PyObject*)config_enum);
-    PyModule_AddObject(mod, "Modifier", (PyObject*)modifier_enum);
-    PyModule_AddObject(mod, "Orient", (PyObject*)orient_enum);
+    PyModule_AddObject(mod, "Config", (PyObject*)st->config_enum);
+    PyModule_AddObject(mod, "Modifier", (PyObject*)st->modifier_enum);
+    PyModule_AddObject(mod, "Orient", (PyObject*)st->orient_enum);
     PyModule_AddObject(mod, "Symbol", (PyObject*)&zbarSymbol_Type);
     PyModule_AddObject(mod, "SymbolSet", (PyObject*)&zbarSymbolSet_Type);
     PyModule_AddObject(mod, "SymbolIter", (PyObject*)&zbarSymbolIter_Type);
@@ -244,26 +315,51 @@ initzbar (void)
     PyModule_AddObject(mod, "Scanner", (PyObject*)&zbarScanner_Type);
 
     for(ei = 0; ei < ZBAR_ERR_NUM; ei++)
-        if(zbar_exc[ei])
-            PyModule_AddObject(mod, exc_names[ei] + 5, zbar_exc[ei]);
+        if(st->zbar_exc[ei])
+            PyModule_AddObject(mod, exc_names[ei] + 5, st->zbar_exc[ei]);
 
     /* add constants */
     PyObject *dict = PyModule_GetDict(mod);
-    color_enum[ZBAR_SPACE] =
+    st->color_enum[ZBAR_SPACE] =
         zbarEnumItem_New(dict, NULL, ZBAR_SPACE, "SPACE");
-    color_enum[ZBAR_BAR] =
+    st->color_enum[ZBAR_BAR] =
         zbarEnumItem_New(dict, NULL, ZBAR_BAR, "BAR");
 
     const enumdef *item;
     for(item = config_defs; item->strval; item++)
-        zbarEnum_Add(config_enum, item->intval, item->strval);
+        zbarEnum_Add(st->config_enum, item->intval, item->strval);
     for(item = modifier_defs; item->strval; item++)
-        zbarEnum_Add(modifier_enum, item->intval, item->strval);
+        zbarEnum_Add(st->modifier_enum, item->intval, item->strval);
     for(item = orient_defs; item->strval; item++)
-        zbarEnum_Add(orient_enum, item->intval, item->strval);
+        zbarEnum_Add(st->orient_enum, item->intval, item->strval);
 
     PyObject *tp_dict = zbarSymbol_Type.tp_dict;
     for(item = symbol_defs; item->strval; item++)
-        zbarEnumItem_New(tp_dict, symbol_enum, item->intval, item->strval);
-    symbol_NONE = zbarSymbol_LookupEnum(ZBAR_NONE);
+        zbarEnumItem_New(tp_dict, st->symbol_enum, item->intval, item->strval);
+    st->symbol_NONE = zbarSymbol_LookupEnum(ZBAR_NONE);
+
+#if PY_MAJOR_VERSION >= 3
+    return mod;
+#endif
+}
+
+
+PyObject*
+zbarErr_Set (PyObject *self)
+{
+    const void *zobj = ((zbarProcessor*)self)->zproc;
+    zbar_error_t err = _zbar_get_error_code(zobj);
+
+    struct module_state *st = GETMODSTATE();
+
+    if(err == ZBAR_ERR_NOMEM)
+        PyErr_NoMemory();
+    else if(err < ZBAR_ERR_NUM) {
+        PyObject *type = st->zbar_exc[err];
+        assert(type);
+        PyErr_SetObject(type, self);
+    }
+    else
+        PyErr_SetObject(st->zbar_exc[0], self);
+    return(NULL);
 }
